@@ -3,29 +3,26 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
+ * The contents of this file constitute Original Code as defined in and
+ * are subject to the Apple Public Source License Version 1.1 (the
+ * "License").  You may not use this file except in compliance with the
+ * License.  Please obtain a copy of the License at
+ * http://www.apple.com/publicsource and read it before using this file.
  * 
- * This file contains Original Code and/or Modifications of Original Code
- * as defined in and that are subject to the Apple Public Source License
- * Version 2.0 (the 'License'). You may not use this file except in
- * compliance with the License. Please obtain a copy of the License at
- * http://www.opensource.apple.com/apsl/ and read it before using this
- * file.
- * 
- * The Original Code and all software distributed under the License are
- * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * This Original Code and all software distributed under the License are
+ * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
- * Please see the License for the specific language governing rights and
- * limitations under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
+ * License for the specific language governing rights and limitations
+ * under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
 /*
- * Copyright (c) 1999 Apple Computer, Inc.  All rights reserved.
+ * Copyright (c) 1999-2003 Apple Computer, Inc.  All rights reserved.
  *
- *  DRI: Josh de Cesare
+ *  DRI: Dave Radcliffe
  *
  */
 
@@ -90,6 +87,11 @@
 #define kIntnVPRVectorMask     (0x000000FF)
 #define kIntnVPRVectorShift    (0)
 
+#define LWBRX(a)	(accessBigEndian ? *(UInt32 *)(a)       : lwbrx(a))
+//#define STWBRX(v,a) (accessBigEndian ? (*(UInt32 *)(a) = (v)) : (stwbrx((v),(a))))
+//The compiler doesn't like the ? : form of the macro for STWBRX.  Don't ask me why.
+#define STWBRX(v,a) if (accessBigEndian)  (*(UInt32 *)(a) = (v)); else stwbrx((v),(a))
+
 class AppleMPICInterruptController : public IOInterruptController
 {
   OSDeclareDefaultStructors(AppleMPICInterruptController);
@@ -102,6 +104,7 @@ private:
   OSSymbol               *interruptControllerName;
   IOService              *parentNub;
   long                   *senses;
+  bool					 isHostMPIC, accessBigEndian;
 
   // callPlatformFunction symbols
   const OSSymbol 	*mpic_dispatchIPI;
@@ -117,6 +120,7 @@ private:
   
 public:
   virtual bool start(IOService *provider);
+  virtual bool matchPropertyTable(OSDictionary * table);
   
   virtual IOReturn getInterruptType(IOService *nub, int source,
 				    int *interruptType);
